uniform vec3 EyePos;        // Object-space camera position

// gl_TexCoord[0] = Normal

#ifdef _VERTEX_

void main()
{
    // Calculate output position
    gl_Position = ftransform();
    gl_TexCoord[0].xyz = gl_Vertex.xyz;
}

#else

void main()
{
    // Init atmosphere parameters
    planRadius2 = Radiuses.x * Radiuses.x;
    mieG2  = AtmoParams2.r * AtmoParams2.r;
    atmoH2 = Radiuses.y * Radiuses.y - planRadius2;
    atmoH  = sqrt(atmoH2);

    // Calculate precise fragment position
    float FragR   = Radiuses.y * 0.999;
    vec3  FragPos = normalize(gl_TexCoord[0].xyz) * FragR;

    // Calculate eye vector in object space
    //vec3  eyeVec = normalize(FragPos - EyePos);
    vec3  eyeVec = FragPos - EyePos;
    float eyeVecLength = length(eyeVec);
    eyeVec /= eyeVecLength;

    // Calculate fragment and eye parameters for atmosphere
    float EyeH;
    float EyeR  = length(EyePos);
    float EyeMu = dot(EyePos, eyeVec) / EyeR;

    // if EyePos in space, move it to nearest intersection of ray with top atmosphere boundary
    float d = -EyeR * EyeMu - sqrt(EyeR * EyeR * (EyeMu * EyeMu - 1.0) + Radiuses.y * Radiuses.y);
    if (d > 0.0)
    {
        EyeMu = (EyeR * EyeMu + d) / Radiuses.y;
        EyeR = Radiuses.y;
        EyeH = 1.0;
    }
    else
        EyeH = (EyeR - Radiuses.x) / Radiuses.z;

    // Atmospheric attenuation along ray from the atmosphere top boundary to the viewer
    //gl_FragColor.rgb = vec3(1.0) - transmittanceWithShadow(EyeR, EyeH, EyeMu);
    //gl_FragColor.rgb = vec3(1.0) - transmittance(EyeH, EyeMu);
    gl_FragColor.rgb = vec3(1.0) - transmittanceAnalytic(EyeR, EyeMu, eyeVecLength);
    gl_FragColor.a = 0.0;
}

#endif
