uniform vec4    FaceParams;  // (x0, y0, size, face)
uniform vec4    Scale;       // (xStep, yStep, height, offset)
uniform sampler2D   BumpMap;

#ifdef _VERTEX_

void main()
{
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}

#else

vec3    GetSurfacePoint(vec2 uv)
{
    vec2 spherical;
    spherical = uv * FaceParams.z + FaceParams.xy;
    vec3 p = normalize(vec3(spherical, 1.0));
    if (FaceParams.w == 0.0)
        return vec3(-p.z, -p.y,  p.x);  // neg_x
    else if (FaceParams.w == 1.0)
        return vec3( p.z, -p.y, -p.x);  // pos_x
    else if (FaceParams.w == 2.0)
        return vec3( p.x, -p.z, -p.y);  // neg_y
    else if (FaceParams.w == 3.0)
        return vec3( p.x,  p.z,  p.y);  // pos_y
    else if (FaceParams.w == 4.0)
        return vec3(-p.x, -p.y, -p.z);  // neg_z
    else
        return vec3( p.x, -p.y,  p.z);  // pos_z
}

void main()
{
    vec2 dx = vec2(Scale.x, 0.0);
    vec2 dy = vec2(0.0, Scale.y);

    float h = (texture2D(BumpMap, gl_TexCoord[0].xy).r * Scale.y - Scale.z) + 1.0;
    vec3  p = GetSurfacePoint(gl_TexCoord[0].xy);
    vec3  t = p - GetSurfacePoint(gl_TexCoord[0].xy + dy);

    gl_FragData[0].xyz = p * h;     // position
    gl_FragData[1].xyz = p;         // normal
    gl_FragData[2].xy  = gl_TexCoord[0].xy;             // uv
    gl_FragData[3].xyz = normalize(t - dot(t, p) * p);  // tangent
}

#endif
