//attribute  vec3  gl_Vertex;           // Sprite center world position
//attribute  vec4  gl_Color;            // Sprite color, Sprite vertex Id
//attribute  vec2  gl_MultiTexCoord0;   // Radius, Luminosity

uniform sampler2D Texture;
uniform vec3  Param;    // GalMagnCoef, GalMagnMin, GalaxyParticleSize
uniform vec3  Param2;   // GalMaxParticleRadius_pix, 1.0 / (GalMaxParticleRadius_pix - 1.0), cam.PixelSize

#ifdef _VERTEX_

out vec2  TexCoord;
out vec4  Color;

void main()
{
    vec4  viewPos = gl_ModelViewMatrix * gl_Vertex;
    float Dist2 = dot(viewPos.xyz, viewPos.xyz);
    float Dist = sqrt(Dist2);

    float mag = Param.x * gl_MultiTexCoord0.y / Dist2;
    float RadiusPix = gl_MultiTexCoord0.x / Dist * Param2.z;
    float fade = clamp(20.0 * (mag - Param.y), 0.0, 1.0) *
                 clamp((Param2.x - RadiusPix) * Param2.y, 0.0, 1.0);

    Color = vec4(gl_Color.rgb * fade, 0.0);

    if (mag > 1.0) mag = 1.0 + log(mag);
    float spriteSize = Param.z * Dist * sqrt(mag);

    TexCoord.x = mod(floor(gl_Color.a * 4.0), 2.0);
    TexCoord.y = mod(floor(gl_Color.a * 2.0), 2.0);

    viewPos.xy += spriteSize * (2.0 * TexCoord.xy - 1.0);
    gl_Position = gl_ProjectionMatrix * viewPos;
}

#else

in vec2  TexCoord;
in vec4  Color;

void main()
{
    gl_FragColor = Color * texture2D(Texture, TexCoord);
}

#endif
