//attribute  vec3  gl_Vertex;           // Sprite center world position
//attribute  vec4  gl_Color;            // Sprite color, Sprite vertex Id
//attribute  vec2  gl_MultiTexCoord0;   // Radius, Luminosity

uniform sampler2D Texture;
uniform vec4      Param;    // GalMagnCoef, GalMagnMin, GlowMagnCoef, cam.PixelSize

#ifdef _VERTEX_

out vec2  TexCoord;
out vec4  Color;

void main()
{
    vec4  viewPos   = gl_ModelViewMatrix * gl_Vertex;
    vec3  viewUpS   = gl_ModelViewMatrixTranspose[2].xyz;
    vec3  viewRight = normalize(cross(viewPos.xyz, viewUpS));
    vec3  viewUp    = normalize(cross(viewRight,   viewPos.xyz));

    float Dist2 = dot(viewPos.xyz, viewPos.xyz);
    float mag = Param.x * gl_MultiTexCoord0.y / Dist2;

    TexCoord.x = mod(floor(gl_Color.a * 4.0), 2.0);
    TexCoord.y = mod(floor(gl_Color.a * 2.0), 2.0);

    vec2  Offset = 2.0 * TexCoord.xy - 1.0;
    viewPos.xyz += gl_MultiTexCoord0.x * (Offset.x * viewRight + Offset.y * viewUp);
    gl_Position = gl_ProjectionMatrix * viewPos;

    float RadiusPix = Param.w * gl_MultiTexCoord0.x / sqrt(Dist2);
    float fade = clamp(20.0 * (mag - Param.y), 0.0, 1.0) *
                 clamp(1.0 - RadiusPix, 0.0, 1.0) * Param.z;
    Color.rgb = clamp(gl_Color.rgb * fade, 0.0, 1.0);
    Color.a = 0.0;
}

#else

in vec2  TexCoord;
in vec4  Color;

void main()
{
    gl_FragColor = Color * texture2D(Texture, TexCoord);
}

#endif
