uniform sampler2D   Tex;
uniform vec2        Size;

#ifdef _VERTEX_

void main()
{
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}

#else

// compress a 4x4 block to YCoCg DXT5 format
// integer version, renders to 4 x int32 buffer
void main()
{
    // read block
    vec3 block[16];
    ExtractColorBlockYCoCg(block, Tex, gl_TexCoord[0].xy, Size);

    // find min and max colors
    vec3 mincol, maxcol;
    FindMinMaxColorsBox(block, mincol, maxcol);

    SelectYCoCgDiagonal(block, mincol.yz, maxcol.yz);

    int scale = ScaleYCoCg(mincol.yz, maxcol.yz);

    // Output CoCg in DXT1 block.
    uvec4 output;
    output.z = EmitEndPointsYCoCgDXT5(mincol.yz, maxcol.yz, scale);
    output.w = EmitIndicesYCoCgDXT5(block, mincol.yz, maxcol.yz);

    InsetYBBox(mincol.x, maxcol.x);

    // Output Y in DXT5 alpha block.
    output.x = EmitAlphaEndPointsYCoCgDXT5(mincol.x, maxcol.x);

    uvec2 indices = EmitAlphaIndicesYCoCgDXT5(block, mincol.x, maxcol.x);
    output.x |= indices.x;
    output.y  = indices.y;

    gl_FragColor = output;
}

#endif
