uniform sampler2D   Tex;
uniform vec2        Size;

#ifdef _VERTEX_

void main()
{
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}

#else

// compress a 4x4 block to LATC format
// integer version, renders to 4 x int32 buffer
void main()
{
    // read block
    vec2 block[16];
    ExtractColorBlockXY(block, Tex, gl_TexCoord[0].xy, Size);

    // find min and max colors
    vec2 mincol, maxcol;
    FindMinMaxColorsBox(block, mincol, maxcol);
    InsetNormalsBBoxLATC(mincol, maxcol);

    uvec4 output;

    // Output Ny as an alpha block.
    output.x = EmitAlphaEndPoints(mincol.y, maxcol.y);

    uvec2 indices = EmitLuminanceIndices(block, mincol.y, maxcol.y);
    output.x |= indices.x;
    output.y  = indices.y;

    // Output Nx as an alpha block.
    output.z = EmitAlphaEndPoints(mincol.x, maxcol.x);

    indices = EmitAlphaIndices(block, mincol.x, maxcol.x);
    output.z |= indices.x;
    output.w  = indices.y;

    gl_FragColor = output;
}

#endif
