//attribute  vec3  gl_Vertex;           // Vertex world position
//attribute  vec2  gl_MultiTexCoord0;   // Texture coordinates

uniform sampler2D Tex1;
uniform sampler2D Tex2;
uniform vec3      Color;
uniform vec4      FadeParams;
uniform vec4      EyePos;

#ifdef _VERTEX_

out vec2  TexCoord;
out vec3  PixPos;

void main()
{
    gl_Position = ftransform();
    TexCoord = gl_MultiTexCoord0.xy;
    PixPos = gl_Vertex.xyz;
}

#else

in vec2  TexCoord;
in vec3  PixPos;

void main()
{
    vec3  ray  = PixPos + EyePos.xyz;
    float dist = length(ray);
    float fade = clamp(dist - FadeParams.x, 0.0, 1.0) *
                 clamp(abs(ray.z / dist) - FadeParams.y, 0.0, 1.0) * FadeParams.z;


    vec3  color = mix(texture2D(Tex1, TexCoord).rgb, texture2D(Tex2, TexCoord).rgb, EyePos.w) * Color.rgb * fade;
    gl_FragColor.rgb = 1.0 - exp(-FadeParams.w * color);

    //gl_FragColor.rgb = fade * Color.rgb * texture2D(Tex1, TexCoord).rgb;
    //gl_FragColor.rgb = 1.0 - exp(-FadeParams.w * gl_FragColor.rgb);

    gl_FragColor.a = max(max(gl_FragColor.r, gl_FragColor.g), gl_FragColor.b);
}

#endif
