//attribute float gl_Vertex;          // Sprite center world position
//attribute float gl_MultiTexCoord0;  // Luminosity
//attribute vec2  gl_Color;           // Color index, vertex id

uniform sampler2D PrecomputedDensityFunc;
uniform vec3      EyePos;
uniform vec4      BulgeColor;
uniform float     Reddening;

#ifdef _VERTEX_

out vec3  VertPos;

void main()
{
    gl_Position = ftransform();
    VertPos = gl_Vertex.xyz;
}

#else

in vec3  VertPos;

void main()
{
    // Calculate parameters to density function
    vec3   vertPos = normalize(VertPos);
    vec3   ray = EyePos + vertPos;                  // ray from the pixel to the eye
    float  depth  = dot(vertPos, normalize(ray));   // half length of the part of the galaxy along the ray
    float  offset = sqrt(1.0 - depth*depth);        // ray's offset from galaxy's center
    depth = min(depth, 0.5*length(ray));            // cut the half length if eye inside the galaxy

    // Now get the visible brightness of pixel
    float density = texture2D(PrecomputedDensityFunc, vec2(offset, depth)).r;

    // Calculate final color
    gl_FragColor = density * density * BulgeColor;

    gl_FragColor.r *= 1.0 - Reddening;
    gl_FragColor.g *= 1.0 - 2.0 * Reddening;
    gl_FragColor.b *= 1.0 - 4.0 * Reddening;
    gl_FragColor = clamp(gl_FragColor, 0.0, 1.0);
}

#endif
