//attribute  vec3  gl_Vertex;           // Sprite center world position
//attribute  vec4  gl_Color;            // Sprite color, Sprite vertex Id
//attribute  vec2  gl_MultiTexCoord0;   // Radius, Luminosity

uniform vec3  Param;    // GalMagnCoef, LabelBrightMin, LabelFade
uniform vec4  Clip;     // Center, Radius^2
uniform vec3  Norm;     // Normal to grid plane, fade
uniform vec4  Color;    // Line color

#ifdef _VERTEX_

out vec4 color;

void main()
{
    vec3  Vector = gl_Vertex.xyz - Clip.xyz;
    float Dist2 = dot(Vector, Vector);

    if (Dist2 <= Clip.w)
    {
        vec4  viewPos = gl_ModelViewMatrix * gl_Vertex;
        float Dist2 = dot(viewPos.xyz, viewPos.xyz);
        float Dist = sqrt(Dist2);

        float mag = (gl_MultiTexCoord0.y / Dist2) * Param.x;
        float fade = clamp((mag - Param.y) * Param.z, 0.0, 1.0);
        color = Color * fade;

        if (mod(floor(gl_Color.a * 4.0), 2.0) == 0.0)
            gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
        else
        {
            vec3 line = gl_Vertex.xyz - dot(Vector, Norm) * Norm;
            gl_Position = gl_ModelViewProjectionMatrix * vec4(line, 1.0);
        }
    }
    else
        gl_Position = vec4(0, 0, 0, 1);
}

#else

in vec4 color;

void main()
{
    gl_FragColor = color;
}

#endif
