//attribute  vec3  gl_Vertex;           // sprite center world position
//attribute  vec4  gl_Color;            // color
//attribute  vec2  gl_MultiTexCoord0;   // texture coordinates
//attribute  vec2  gl_MultiTexCoord1;   // vertex offset
//attribute  vec2  gl_MultiTexCoord2;   // burn, size

uniform sampler2D Texture;
uniform sampler2D PrecomputedDensityFunc;

uniform vec4      BrightZoom;   // (emBright, absBright, Zoom, cam.PixelSize)
uniform vec3      EyePos;
uniform vec4      BulgeColor;
uniform float     BulgeRadius;
uniform vec4      Clip;         // (Center, Radius^2)

#ifdef _VERTEX_

out vec3  VertPos;
out vec2  TexCoord;
out vec4  Color;

void main()
{
    vec3  Vector = gl_Vertex.xyz - Clip.xyz;
    float Dist2 = dot(Vector, Vector);
    float fade = clamp((Clip.w - Dist2) * 100.0, 0.0, 1.0);

    vec4  viewPos   = gl_ModelViewMatrix * gl_Vertex;
    vec3  viewUpS   = gl_ModelViewMatrixTranspose[2].xyz;
    vec3  viewRight = normalize(cross(viewPos.xyz, viewUpS));
    vec3  viewUp    = normalize(cross(viewRight,   viewPos.xyz));
    float spriteSize = BrightZoom.z * gl_MultiTexCoord2.y;

    viewPos.xyz += spriteSize * (gl_MultiTexCoord1.x * viewRight + gl_MultiTexCoord1.y * viewUp);
    gl_Position = gl_ProjectionMatrix * viewPos;

    fade *= gl_Color.a * clamp(- viewPos.z / (spriteSize*BrightZoom.w) - 0.5, 0.0, 1.0);
    Color.rgb = BrightZoom.x * gl_Color.rgb;
    Color.a   = BrightZoom.y * (1.0 - gl_MultiTexCoord2.x);
    Color    *= fade;

    TexCoord = gl_MultiTexCoord0.xy;
    VertPos  = gl_Vertex.xyz;
}

#else

in vec3  VertPos;
in vec2  TexCoord;
in vec4  Color;

void main()
{
    // Calculate parameters to density function
    vec3   ray = normalize(EyePos + VertPos);   // ray from the pixel to the eye

    float  ph = dot(VertPos, ray);
    float  offset = sqrt(dot(VertPos, VertPos) - ph*ph);    // ray's offset from galaxy's center

    float density = 0.0;

    if (offset < BulgeRadius)
    {
        float  eh = dot(EyePos,  ray);
        float  maxHalfDepth = sqrt(BulgeRadius*BulgeRadius - offset*offset);
        float  maxDepth = 2.0 * maxHalfDepth;
        float  pixDepth = max(0.0, min(maxHalfDepth + ph, maxDepth));
        float  eyeDepth = min(maxHalfDepth - eh, maxDepth);

        // Now get the density of bulge glow
        density = texture2D(PrecomputedDensityFunc, vec2(offset, 0.5 * pixDepth)).r;

        if (eyeDepth > 0.0)
            density -= texture2D(PrecomputedDensityFunc, vec2(offset, 0.5 * eyeDepth)).r;

        density = density * density;
    }

    // Calculate final color
    gl_FragColor = (density * density * BulgeColor + Color) * texture2D(Texture, TexCoord);
    //gl_FragColor.rgb = 1.0 - exp(-2.0*gl_FragColor.rgb);
}

#endif
