//attribute  vec3  gl_Vertex;           // sprite center world position
//attribute  vec4  gl_Color;            // color
//attribute  vec2  gl_MultiTexCoord0;   // texture coordinates
//attribute  vec2  gl_MultiTexCoord1;   // vertex offset
//attribute  vec2  gl_MultiTexCoord2;   // burn, size

uniform sampler2D Texture;
uniform vec4      BrightZoom;   // (emBright, absBright, Zoom, cam.PixelSize)
uniform vec4      Clip;         // (Center, Radius^2)

#ifdef _VERTEX_

out vec2  TexCoord;
out vec4  Color;

void main()
{
    vec3  Vector = gl_Vertex.xyz - Clip.xyz;
    float Dist2 = dot(Vector, Vector);
    float fade = clamp((Clip.w - Dist2) * 100.0, 0.0, 1.0);

    vec4  viewPos   = gl_ModelViewMatrix * gl_Vertex;
    vec3  viewUpS   = gl_ModelViewMatrix[2].xyz;
    vec3  viewRight = normalize(cross(viewPos.xyz, viewUpS));
    vec3  viewUp    = normalize(cross(viewRight,   viewPos.xyz));
    float spriteSize = BrightZoom.z * gl_MultiTexCoord2.y;

    viewPos.xyz += spriteSize * (gl_MultiTexCoord1.x * viewRight + gl_MultiTexCoord1.y * viewUp);
    gl_Position = gl_ProjectionMatrix * viewPos;

    fade *= gl_Color.a * clamp(-viewPos.z / (spriteSize*BrightZoom.w) - 0.5, 0.0, 1.0);
    Color.rgb = BrightZoom.x * gl_Color.rgb;
    Color.a   = BrightZoom.y * (1.0 - gl_MultiTexCoord2.x);
    Color    *= fade;

    TexCoord = gl_MultiTexCoord0.xy;
}

#else

in vec2  TexCoord;
in vec4  Color;

void main()
{
    gl_FragColor = Color * texture2D(Texture, TexCoord);
}

#endif
