//attribute  vec3  gl_Vertex;           // Sprite center world position
//attribute  vec4  gl_Color;            // Sprite color, Sprite vertex Id
//attribute  vec2  gl_MultiTexCoord0;   // Radius, Luminosity

uniform vec4  Param;    // GalMagnCoef, GalMagnMin, GalaxyPointParticleSize, Horizon
uniform vec3  Param2;   // GalMaxParticleRadius_pix, 1.0 / (GalMaxParticleRadius_pix - 1.0), cam.PixelSize

#ifdef _VERTEX_

out vec4  Color;

void main()
{
    vec4  viewPos = gl_ModelViewMatrix * gl_Vertex;
    gl_Position = gl_ProjectionMatrix * viewPos;

    float Dist2 = dot(viewPos.xyz, viewPos.xyz);
    float Dist = sqrt(Dist2);
    float HorDist = Dist / Param.w;

    float mag = (gl_MultiTexCoord0.y / Dist2) * Param.x;
    float RadiusPix = gl_MultiTexCoord0.x / Dist * Param2.z;
    float fade = clamp(20.0 * (mag - Param.y), 0.0, 1.0) *
                 clamp((Param2.x - RadiusPix) * Param2.y, 0.0, 1.0);

    Color.rgb = gl_Color.rgb * fade;
    Color.r *= 1.0 - HorDist;
    Color.g *= 1.0 - 2.0 * HorDist;
    Color.b *= 1.0 - 4.0 * HorDist;
    Color.a = 1.0;
    Color = clamp(Color, 0.0, 1.0);

    if (isinf(mag))
        Color.rgb = vec3(0.5, 0.0, 0.2);

    if (mag > 1.0) mag = 1.0 + log(mag);
    gl_PointSize = Param.z * sqrt(mag);
    //gl_PointSize = Param.z * sqrt(mag);
}

#else

in vec4  Color;

void main()
{
    gl_FragColor = Color;
}

#endif
