//attribute  vec3  gl_Vertex;           // Vertex world position
//attribute  vec2  gl_MultiTexCoord0;   // Texture coordinates

uniform sampler2D Texture;
uniform vec4      EyePos;
uniform vec4      FadeParams;
uniform vec4      Clip;

#ifdef _VERTEX_

out vec2  TexCoord;
out vec3  PixPos;

void main()
{
    gl_Position = ftransform();
    TexCoord = gl_MultiTexCoord0.xy;
    PixPos = gl_Vertex.xyz;
}

#else

in vec2  TexCoord;
in vec3  PixPos;

void main()
{
    vec3  ray  = PixPos + EyePos.xyz;
    float dist = length(ray);
    vec3  Vector = PixPos - Clip.xyz;
    float dist2 = dot(Vector, Vector);
    float fade = clamp((dist - FadeParams.x) * FadeParams.y, 0.0, 1.0) *
                 clamp((abs(ray.z / dist) - FadeParams.z) * FadeParams.w, 0.0, 1.0) *
                 clamp((Clip.w - dist2) * 1000.0, 0.0, 1.0);
    gl_FragColor = fade * texture2D(Texture, TexCoord);
    gl_FragColor.rgb *= EyePos.w;
}

#endif
