//attribute  vec3  gl_Vertex;           // sprite center world position
//attribute  vec4  gl_Color;            // color
//attribute  vec2  gl_MultiTexCoord0;   // texture coordinates
//attribute  vec2  gl_MultiTexCoord1;   // vertex offset
//attribute  vec2  gl_MultiTexCoord2;   // burn, size
//attribute  vec4  gl_MultiTexCoord3;   // lights

uniform sampler2D Texture;
uniform vec4      BrightZoom;   // (emBright, Fade, Zoom, cam.PixelSize)

#ifdef _VERTEX_

out vec2  TexCoord;
out vec4  Color;

void main()
{
    vec4  viewPos   = gl_ModelViewMatrix * gl_Vertex;
    vec3  viewUpS   = gl_ModelViewMatrix[2].xyz;
    vec3  viewRight = normalize(cross(viewPos.xyz, viewUpS));
    vec3  viewUp    = normalize(cross(viewRight,   viewPos.xyz));
    float spriteSize = BrightZoom.z * gl_MultiTexCoord2.y;

    viewPos.xyz += spriteSize * (gl_MultiTexCoord1.x * viewRight + gl_MultiTexCoord1.y * viewUp);
    gl_Position = gl_ProjectionMatrix * viewPos;

    Color.rgb = BrightZoom.x * gl_Color.rgb;
    Color.a   = (1.0 - gl_MultiTexCoord2.x);
    Color    *= BrightZoom.y * gl_Color.a * clamp(-viewPos.z / (spriteSize*BrightZoom.w) - 0.5, 0.0, 1.0);

    TexCoord = gl_MultiTexCoord0.xy;
}

#else

in vec2  TexCoord;
in vec4  Color;

void main()
{
    gl_FragColor = Color * texture2D(Texture, TexCoord).a;
}

#endif
