//attribute  vec3  gl_Vertex;           // sprite center world position
//attribute  vec4  gl_Color;            // color
//attribute  vec2  gl_MultiTexCoord0;   // texture coordinates
//attribute  vec2  gl_MultiTexCoord1;   // vertex offset
//attribute  vec2  gl_MultiTexCoord2;   // burn, size
//attribute  vec4  gl_MultiTexCoord3;   // lights

uniform sampler2D Texture;
uniform vec4      BrightZoom;   // (emBright, Fade, Zoom, cam.PixelSize)
uniform vec4      Clip;         // (clipFocus, clipRadius2)
uniform float     BumpHeight;
uniform vec4      DustColor;
uniform vec3      Light0Pos;

#ifdef _VERTEX_

out vec3  VertPos;
out vec2  TexCoord;
out vec3  Normal;
out vec4  Color;
out vec3  BurnFade;

void main()
{
    vec3  Vector = gl_Vertex.xyz - Clip.xyz;
    float Dist2 = dot(Vector, Vector);
    BurnFade.z = clamp((Clip.w - Dist2) * 10.0, 0.0, 1.0);

    vec4  viewPos   = gl_ModelViewMatrix * gl_Vertex;
    vec3  viewUpS   = gl_ModelViewMatrix[2].xyz;
    vec3  viewRight = normalize(cross(viewPos.xyz, viewUpS));
    vec3  viewUp    = normalize(cross(viewRight,   viewPos.xyz));
    float spriteSize = BrightZoom.z * gl_MultiTexCoord2.y;

    vec3  Offset = gl_MultiTexCoord1.x * viewRight + gl_MultiTexCoord1.y * viewUp;
    viewPos.xyz += spriteSize * Offset;
    gl_Position = gl_ProjectionMatrix * viewPos;

    Normal = BumpHeight * gl_ModelViewProjectionMatrix[2].xyz - Offset;

    Color.rgb = BrightZoom.x * gl_Color.rgb;
    Color.a   = 0.001 * (1.0 - gl_MultiTexCoord2.x);
    Color    *= BrightZoom.y * gl_Color.a * clamp(-viewPos.z / (spriteSize*BrightZoom.w) - 0.5, 0.0, 1.0);
    BurnFade.x = gl_MultiTexCoord2.x;
    BurnFade.y = BrightZoom.y;

    TexCoord = gl_MultiTexCoord0.xy;
    VertPos  = gl_Vertex.xyz;
}

#else

in vec3  VertPos;
in vec2  TexCoord;
in vec3  Normal;
in vec4  Color;
in vec3  BurnFade;

float HG(float cosa) // Henyey-Greenstein phase function
{
    return 1.0;
    const float g = 0.6;
    return 0.5 * (1.0 - g*g) / pow(1.0 + g*g - 2.0*g*cosa, 1.5);
}

void main()
{
    vec3  light0Pos = Light0Pos - VertPos;

    float bright0 = gl_LightSource[0].diffuse.a * BurnFade.z / (dot(light0Pos, light0Pos) + 0.05);

    vec4  texData = texture2D(Texture, TexCoord);
    float diffColor = texData.a;

    if (BurnFade.x > 0.1)
    {
        gl_FragColor = diffColor * Color * bright0;
    }
    else
    {
        vec3   norm = normalize(Normal);
        vec4   bright = vec4(0);
        float  cosa,absLit;

        cosa = dot(norm, normalize(light0Pos));
        absLit = bright0 * HG(cosa);
        bright.rgb += gl_LightSource[0].diffuse.rgb * absLit;
        bright.a += bright0;

        bright.a *= Color.a;
        gl_FragColor = diffColor * DustColor * bright * BurnFade.y;
    }
}

#endif
