//attribute vec4  gl_Color;           // Color
//attribute vec2  gl_SecondaryColor;  // Sprite texture coordinates
//attribute vec2  gl_MultiTexCoord0;  // Radius, Luminosity
//attribute vec3  gl_MultiTexCoord1;  // Sprite center old world position
//attribute vec3  gl_Vertex;          // Sprite center world position

uniform sampler2D Texture;

#ifdef _VERTEX_

out vec3 TexCoord;
out vec4 Color;

void main()
{
    TexCoord.xy = gl_SecondaryColor.xy;
    Color = gl_Color;

    vec4 viewPosN = gl_ModelViewMatrix * gl_Vertex;
    vec4 viewPosO = gl_ModelViewMatrix * vec4(gl_MultiTexCoord1.xyz, 1.0);
    vec4 viewPos = (TexCoord.x == 0.0) ? viewPosN : viewPosO;

    float spriteSize = gl_MultiTexCoord0.x;

    vec2 Offset = 1.0 - TexCoord.xy * 2.0;
    vec3 Ray = viewPosN.xyz - viewPosO.xyz;

    if (dot(Ray, Ray) <= 1e-6)
    {
        viewPos.xy += Offset * spriteSize;
        TexCoord.z = 0.0;
    }
    else
    {
        //float RayLen = 0.5 * length(viewPosN.xy - viewPosO.xy) / ((1.0 + abs(viewPosN.z - viewPosO.z)) * spriteSize);
        float RayLen = 0.5 * length(viewPosN.xy - viewPosO.xy);

        //spriteSize /= RayLen + 1.0;
        //spriteSize /= RayLen*RayLen + 1.0;
        TexCoord.z = RayLen / (RayLen + 1.0);

        vec3 Tangent = normalize(cross(Ray, viewPosN.xyz));
        mat2x2 Mat = mat2x2(Tangent.y, -Tangent.x, Tangent.x, Tangent.y);
        viewPos.xy += (Mat * Offset) * spriteSize;
    }

    gl_Position = gl_ProjectionMatrix * viewPos;
}

#else

in vec3 TexCoord;
in vec4 Color;

void main()
{
    vec2 texCoord;
    texCoord.y = TexCoord.y;
    if (abs(2.0 * TexCoord.x - 1.0) < TexCoord.z)
        texCoord.x = 0.5;
    else
    {
        if (TexCoord.x > 0.5)
            texCoord.x = (TexCoord.x - TexCoord.z) / (1.0 - TexCoord.z);
        else
            texCoord.x = TexCoord.x / (1.0 - TexCoord.z);
    }

    gl_FragColor = Color * texture2D(Texture, texCoord);
}

#endif
