//attribute float gl_Vertex;          // Sprite center world position
//attribute float gl_MultiTexCoord0;  // Luminosity
//attribute vec2  gl_Color;           // Color index, vertex id

uniform sampler1D ColorMap;
uniform sampler2D Texture;
uniform vec4  Param;    // StarBrightCoef, StarBrightMin, StarParticleSize, StarParticleFadeOut
uniform vec4  Clip;     // Center, Radius^2

#ifdef _VERTEX_

out vec2 TexCoord;
out vec2 ColBright;

void main()
{
    vec3  Vector = gl_Vertex.xyz - Clip.xyz;
    float Dist2 = dot(Vector, Vector);
    if (Dist2 <= Clip.w)
    {
        vec4  viewPos = gl_ModelViewMatrix * gl_Vertex;

        Dist2 = dot(viewPos.xyz, viewPos.xyz);
        float mag = (exp(gl_MultiTexCoord0.x) / Dist2) * Param.x;

        ColBright.x = gl_Color.x;
        ColBright.y = clamp(Param.w*(mag - Param.y), 0.0, 1.0);

        if (mag > 1.0)  mag = 1.0 + log(mag);
        float spriteSize = Param.z * sqrt(Dist2 * mag);

        TexCoord.x = mod(floor(gl_Color.y * 4.0), 2.0);
        TexCoord.y = mod(floor(gl_Color.y * 2.0), 2.0);

        viewPos.xy += spriteSize * (TexCoord * 2.0 - 1.0);
        gl_Position = gl_ProjectionMatrix * viewPos;
    }
    else
        gl_Position = vec4(0, 0, 0, 1);
}

#else

in vec2 TexCoord;
in vec2 ColBright;

void main()
{
    vec4 Color = texture1D(ColorMap, ColBright.x);
    gl_FragColor.rgb = ColBright.y * Color.rgb * texture2D(Texture, TexCoord).rgb;
    gl_FragColor.a = ColBright.y;
}

#endif
