//attribute float gl_Vertex;          // Sprite center world position
//attribute float gl_MultiTexCoord0;  // Luminosity
//attribute vec2  gl_Color;           // Color index, vertex id

uniform vec4  Param;    // StarBrightCoef, StarBrightMin, StarParticleSize, StarParticleFadeOut
uniform vec4  Clip;     // Center, Radius^2
uniform vec4  NormFade; // Normal to grid plane, fade
uniform vec4  Color;    // Line color

#ifdef _VERTEX_

out vec4 color;

void main()
{
    vec3  Vector = gl_Vertex.xyz - Clip.xyz;
    float Dist2 = dot(Vector, Vector);

    if (Dist2 <= Clip.w)
    {
        vec4  viewPos = gl_ModelViewMatrix * gl_Vertex;

        Dist2 = dot(viewPos.xyz, viewPos.xyz);
        float mag = (exp(gl_MultiTexCoord0.x) / Dist2) * Param.x;

        float fade = clamp(Param.w * (mag - Param.y), 0.0, 1.0) * NormFade.w;
        color = Color * fade;

        if (mod(floor(gl_Color.y * 4.0), 2.0) == 0.0)
            gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
        else
        {
            vec3 line = gl_Vertex.xyz - dot(Vector, NormFade.xyz) * NormFade.xyz;
            gl_Position = gl_ModelViewProjectionMatrix * vec4(line, 1.0);
        }
    }
    else
        gl_Position = vec4(0, 0, 0, 1);
}

#else

in vec4 color;

void main()
{
    gl_FragColor = color;
}

#endif
