//attribute float gl_Vertex;          // Sprite center world position
//attribute float gl_MultiTexCoord0;  // Luminosity
//attribute vec2  gl_Color;           // Color index, vertex id

uniform sampler1D ColorMap;
uniform sampler2D Texture;
uniform vec4  Param;    // StarBrightCoef, StarBrightMin, StarParticleSize, StarParticleFadeOut
uniform vec4  Clip;     // Center, Radius^2
uniform vec3  Norm;     // Normal to grid plane

varying vec3 TexCoord;
varying vec2 ColBright;

#ifdef _VERTEX_

void main()
{
    vec3  Vector = gl_Vertex.xyz - Clip.xyz;
    float Dist2 = dot(Vector, Vector);
    if (Dist2 <= Clip.w)
    {
        TexCoord.x = mod(floor(gl_Color.y * 4.0), 2.0);
        TexCoord.y = mod(floor(gl_Color.y * 2.0), 2.0);

        vec3 line = gl_Vertex.xyz - dot(Vector, Norm) * Norm;
        vec4 viewPosN = gl_ModelViewMatrix * gl_Vertex;
        vec4 viewPosO = gl_ModelViewMatrix * vec4(line, 1.0);
        vec4 viewPos;
        if (TexCoord.x == 0.0) viewPos = viewPosN;
        else                   viewPos = viewPosO;

        float Dist2 = dot(viewPos.xyz, viewPos.xyz);
        float mag = Param.x * exp(gl_MultiTexCoord0.x) / Dist2;

        ColBright.x = gl_Color.x;
        ColBright.y = clamp(Param.w*(mag - Param.y), 0.0, 1.0);

        if (mag > 1.0)  mag = 1.0 + log(mag);
        float spriteSize = Param.z * sqrt(Dist2 * mag);

        vec2 Offset = 1.0 - TexCoord.xy * 2.0;
        vec3 Ray = viewPosN.xyz - viewPosO.xyz;

        //if ((dot(Ray, Ray) <= 1e-6) || (abs(viewPosN.z - viewPosO.z) <= 1e-4))
        if (dot(Ray, Ray) <= 1e-6)
        {
            viewPos.xy += Offset * spriteSize;
            TexCoord.z = 0.0;
        }
        else
        {
            //float RayLen = 0.5 * length(viewPosN.xy - viewPosO.xy) / ((1.0 + abs(viewPosN.z - viewPosO.z)) * spriteSize);
            float RayLen = 0.5 * length(viewPosN.xy - viewPosO.xy);

            spriteSize /= RayLen + 1.0;
            //spriteSize /= RayLen*RayLen + 1.0;
            TexCoord.z = RayLen / (RayLen + 1.0);

            /*if (dot(Ray, viewPosN.xyz) <= 1e-3)
            {
                viewPos.xy += Offset * spriteSize;
                TexCoord.z = 0.0;
            }
            else*/
            {
                vec3 Tangent = normalize(cross(Ray, viewPosN.xyz));
                mat2x2 Mat = mat2x2(Tangent.y, -Tangent.x, Tangent.x, Tangent.y);
                viewPos.xy += (Mat * Offset) * spriteSize;
            }
        }

        gl_Position = gl_ProjectionMatrix * viewPos;
    }
    else
        gl_Position = vec4(0, 0, 0, 1);
}

#else

void main()
{
    vec2 texCoord;
    texCoord.y = TexCoord.y;
    if (abs(2.0 * TexCoord.x - 1.0) < TexCoord.z)
        texCoord.x = 0.5;
    else
    {
        if (TexCoord.x > 0.5)
            texCoord.x = (TexCoord.x - TexCoord.z) / (1.0 - TexCoord.z);
        else
            texCoord.x = TexCoord.x / (1.0 - TexCoord.z);
    }

    vec4 Color = texture1D(ColorMap, ColBright.x);
    gl_FragColor.rgb = ColBright.y * Color.rgb * texture2D(Texture, texCoord).rgb;
    gl_FragColor.a = 0.0;
}

#endif
