#ifdef _VERTEX_

void main()
{
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}

#else

vec4  ColorMapSelena(vec3 point, float height, float slope)
{
    vec4 color;

    // Assign a material
    noiseOctaves = (cloudsStyle == 1.0) ? 5.0 : 12.0;
    float mat, dist;

    if (tidalLock == 0.0)
        //mat = 1.0f;
        mat = height;
    else
    {
        float latitude = 1.0 - point.x;
        latitude += 0.15 * (Fbm(point * 0.7 + Randomize) - 1.0);
        mat = mix(climateTropic, climatePole, latitude);
    }

    //if (height <= 0.05)
    //    mat = 0.0;

    // Color texture distortion
    noiseOctaves = 15.0;
    dist = 1.5 * floor(2.0 * DistFbm(point * 0.002 * colorDistFreq, 2.0));
    mat += colorDistMagn * dist;

    // Color texture variation
    noiseOctaves = 8.0;
    //float vary = 4.0 * abs(Fbm(point * 15 * colorDistFreq));
    vec3 p = point * 10.0 * colorDistFreq;
    p += FbmVec3(p);
    float vary = Cell2Noise(p) * 4.0;

    //color = GetSurfaceColorAtlas(saturate(mat), slope, vary);
    color = GetSurfaceColorAtlasSmooth(saturate(mat), slope, vary);

    // Ice cracks
    vec4 colorMult = vec4(1.0);
    if (cracksOctaves > 0.0)
    {
        // Rim height distortion
        noiseH          = 0.5;
        noiseLacunarity = 2.218281828459;
        noiseOffset     = 0.8;
        noiseOctaves    = 4.0;
        float dunes = 0.7 * max(Fbm(point * dunesFreq) + 0.7, 0.0);
        noiseOctaves    = 6.0;
        float crack = CrackColorNoise(point);// * (1.0 + 0.3 * dunes);
        colorMult = mix(vec4(0.67, 0.58, 0.36, 0.00), colorMult, crack);
        //slope = 1.0 - smoothstep(0.0, 0.05, crack);
    }

    color *= colorMult;

    // Rayed craters
    float craterRayedSqrtDensity = craterSqrtDensity * sqrt(craterRayedFactor);
    if (craterRayedSqrtDensity > 0.05)
    {
        noiseH           = 0.5;
        noiseLacunarity  = 2.218281828459;
        noiseOffset      = 0.8;
        noiseOctaves     = 3;  // Craters roundness distortion
        craterDistortion = 1.0;
        craterRoundDist  = 0.03;
        heightFloor      = -0.5;
        heightPeak       = 0.6;
        heightRim        = 1.0;
        float craterRayedOctaves = floor(craterOctaves * craterRayedFactor);
        float crater = RayedCraterColorNoise(point, craterFreq, craterRayedSqrtDensity, craterRayedOctaves);
        color.rgb += vec3(0.5 * crater);
    }

    // Make driven hemisphere darker
    if (drivenDarkening != 0.0)
    {
        noiseOctaves = 3;
        float z = -point.z * sign(drivenDarkening);
        z += 0.2 * Fbm(point * 1.63);
        z = saturate(1.0 - z);
        z *= z;
        color.rgb *= mix(1.0 - abs(drivenDarkening), 1.0, z);
    }

    return color;
}

void main()
{
    vec3  point = GetSurfacePoint();
    vec2  texCoord = gl_TexCoord[0].xy * scaleParams.z + scaleParams.xy;
    vec4  bumpData = texture2D(NormalMap, texCoord);
    vec3  norm = 2.0 * bumpData.xyz - 1.0;
    float slope = clamp(1.0 - pow(norm.z, 6.0), 0.0, 1.0);
    float height = bumpData.w;
    gl_FragColor = ColorMapSelena(point, height, slope);
}

#endif
