#ifdef _VERTEX_

void main()
{
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}

#else

float   HeightMapSelena(vec3 point)
{
    float massif, global;
    vec3  distort;
    vec3  p = point * mainFreq + Randomize;

    // Global landscape
    noiseOctaves = 5;
    distort = 0.35 * FbmVec3(p * 2.37);
    noiseOctaves = 4;
    distort += 0.005 * (1.0 - abs(FbmVec3(p * 132.3)));
    global = 0.6 * (1.0 - Cell3Noise(p + distort));

    // Venus-like structure
    float venus = 0.0;
    if (venusMagn > 0.05)
    {
        noiseOctaves = 4;
        distort = FbmVec3(point * 0.3) * 1.5;
        noiseOctaves = 6;
        venus = Fbm((point + distort) * venusFreq) * venusMagn;
    }
    global += venus;

    // Mare floor landscape
    float mareFloor = 0.0;

    // Mare
    float mare = global;
    if (mareSqrtDensity > 0.05)
    {
        craterDistortion = 1.0;
        noiseOctaves = 6;  // Mare roundness distortion
        mare = MareNoise(point, global, mareFloor);
    }

    // Old craters
    float crater = 0.0;
    if (craterSqrtDensity > 0.05)
    {
        noiseOctaves = 3;  // Craters roundness distortion
        craterDistortion = 1.0;
        craterRoundDist  = 0.03;
        heightFloor = -0.1;
        heightPeak  = 0.6;
        heightRim   = 1.0;
        crater = CraterNoise(point, craterMagn, craterFreq, craterSqrtDensity, craterOctaves);
        noiseOctaves = 11.0;
        noiseH       = 1.0;
        noiseOffset  = 1.0;
        crater = 0.5 * RidgedMultifractalDetail(point * 0.5 * montesFreq + Randomize, 2.0, 0.25 * crater);
    }

    float height = mare + crater;

    // Fill mare with lava
    height = max(height - seaLevel, 0.0);

    // Ice cracks
    if (cracksOctaves > 0.0)
    {
        // Rim height distortion
        noiseH          = 0.5;
        noiseLacunarity = 2.218281828459;
        noiseOffset     = 0.8;
        noiseOctaves    = 4.0;
        float dunes = 2 * cracksMagn * (0.2 + dunesMagn * max(Fbm(point * dunesFreq) + 0.7, 0.0));
        noiseOctaves    = 6.0;
        height += CrackNoise(point, dunes);
    }

    // Rayed craters
    float craterRayedSqrtDensity = craterSqrtDensity * sqrt(craterRayedFactor);
    if (craterRayedSqrtDensity > 0.05)
    {
        noiseH           = 0.5;
        noiseLacunarity  = 2.218281828459;
        noiseOffset      = 0.8;
        noiseOctaves     = 3;  // Craters roundness distortion
        craterDistortion = 1.0;
        craterRoundDist  = 0.03;
        heightFloor      = -0.5;
        heightPeak       = 0.6;
        heightRim        = 1.0;
        float craterRayedOctaves = floor(craterOctaves * craterRayedFactor);
        float craterRayedMagn = craterMagn * pow(0.62, craterOctaves - craterRayedOctaves);
        crater = RayedCraterNoise(point, craterRayedMagn, craterFreq, craterRayedSqrtDensity, craterRayedOctaves);
        //noiseOctaves = 11.0;
        //noiseH       = 1.0;
        //noiseOffset  = 1.0;
        //crater = 0.5 * RidgedMultifractalDetail(point * 0.3 * montesFreq + Randomize, 2.0, 0.25 * crater);
        height += crater;
    }

    // Fill mare with lava
    height = max(height, 0.0);

    return height;
}

void main()
{
    vec3  point = GetSurfacePoint();
    float height = HeightMapSelena(point);
    gl_FragColor = vec4(height);
}

#endif
