#ifdef _VERTEX_

void main()
{
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}

#else

vec4  ColorMapTerra(vec3 point, float height, float slope)
{
    vec3  p = point * mainFreq + Randomize;
    vec4  col;
    noiseOctaves = 6;
    vec3  distort = p * 2.3 + 3.5 * FbmVec3(p * 0.6);
    vec2  cell = Cell3Noise2Color(distort, col);
    float biome = col.r;
    float biomeScale = saturate(2.0 * (pow(abs(cell.y - cell.x), 0.7) - 0.05));
    float terrace = col.g;
    float terraceLayers = max(col.b * 10.0 + 3.0, 3.0);

    vec4  colorOverlay;
    if (biome < dunesFraction)
        colorOverlay = vec4(1.0, 1.0, 0.0, 0.0);
    else if (biome < hillsFraction)
        colorOverlay = vec4(0.0, 1.0, 0.0, 0.0);
    else if (biome < hills2Fraction)
        colorOverlay = vec4(0.0, 1.0, 0.5, 0.0);
    else if (biome < canyonsFraction)
        colorOverlay = vec4(1.0, 0.0, 0.0, 0.0);
    else
        colorOverlay = vec4(1.0, 1.0, 1.0, 0.0);

    //noiseOctaves = 4;
    //distort = 0.35 * FbmVec3(p * 23.8);
    //float pseudoRivers = abs(0.5 - Cell3Noise(3.1 * p + distort));
    //pseudoRivers = 1.0 - smoothstep(0.0, 1.0, pseudoRivers * 60.0);

    vec4 color;

    // Assign a climate type
    noiseOctaves = (cloudsStyle == 1.0) ? 5.0 : 12.0;
    float climate, latitude, dist;
    if (tidalLock == 0.0)
    {
        latitude = abs(point.y);
        latitude += 0.15 * (Fbm(point * 0.7 + Randomize) - 1.0);
        latitude = saturate(latitude);
        if (latitude < latTropic - tropicWidth)
            climate = mix(climateTropic, climateEquator, (latTropic - tropicWidth - latitude) / latTropic);
        else if (latitude > latTropic + tropicWidth)
            climate = mix(climateTropic, climatePole, (latitude - latTropic - tropicWidth) / (1.0 - latTropic));
        else
            climate = climateTropic;
    }
    else
    {
        latitude = 1.0 - point.x;
        latitude += 0.15 * (Fbm(point * 0.7 + Randomize) - 1.0);
        climate = mix(climateTropic, climatePole, latitude);
    }

    // Change climate with elevation
    float montHeight = max((height - seaLevel) / (snowLevel - seaLevel), 0.0);
    climate = mix(climate, climatePole, montHeight * montHeight);

    // Beach
    if (height < seaLevel * 1.01) climate = 0.375;
    //float beach = saturate((height / seaLevel - 1.0) * 64.0);
    //climate = mix(0.375, climate, beach);

    // Color texture distortion
    noiseOctaves = 15.0;
    dist = floor(2.0 * DistFbm(point * 0.002 * colorDistFreq, 2.0));
    climate += colorDistMagn * dist;

    // Color texture variation
    noiseOctaves = 8.0;
    p = point * 5.0 * 0.2 * colorDistFreq;
    p += FbmVec3(p);
    float vary = Cell2Noise(p) * 4.0;

    if (biome < dunesFraction)
        slope = 0.0;

    color = GetSurfaceColorAtlasSmooth(climate, slope, vary);

    //if (pseudoRivers > 0.6) color = vec4(0.04, 0.10, 0.20, 1.00);

    // Snow line variation
    noiseOctaves = 8.0;
    climate += 0.25 * montHeight * Fbm(point * 0.5 * colorDistFreq);

    // Ice caps
    if ((latitude > latIceCaps - 0.001) || (climate >= 0.875))
    {
        noiseOctaves = 8.0;
        int vary = int(8.0 * abs(DistFbm(point * 0.5 * colorDistFreq, 4.0))) % 8;
        color = GetSurfaceColorAtlas(1.0, 0.0, vary);
    }

    color = mix(color, colorOverlay * biomeScale, 0.04);

    return color;
}

void main()
{
    vec3  point = GetSurfacePoint();
    vec2  texCoord = gl_TexCoord[0].xy * scaleParams.z + scaleParams.xy;
    vec4  bumpData = texture2D(NormalMap, texCoord);
    vec3  norm = 2.0 * bumpData.xyz - 1.0;
    float slope = clamp(1.0 - pow(norm.z, 6.0), 0.0, 1.0);
    gl_FragColor = ColorMapTerra(point, bumpData.w, slope); 
}

#endif
